@@ -23,10 +23,14 @@
            :dump-image
            :list-fasls
            :*source-directory*
-           :*fasl-directory*))
+           :*fasl-directory*
+           :*started-from-emacs*
+           :define-package))
 
 (cl:in-package :swank-loader)
 
+(defvar *started-from-emacs* nil)
+
 (defvar *source-directory*
   (make-pathname :name nil :type nil
                  :defaults (or *load-pathname* *default-pathname-defaults*))
@@ -43,24 +47,25 @@
   #+lispworks '((swank lispworks) (swank gray))
   #+allegro '((swank allegro) (swank gray))
   #+clisp '(xref metering (swank clisp) (swank gray))
-  #+armedbear '((swank abcl))
+  #+armedbear '((swank abcl) (swank gray))
   #+cormanlisp '((swank corman) (swank gray))
   #+ecl '((swank ecl) (swank gray))
-  #+clasp '((swank clasp) (swank gray))
+  #+clasp '(metering (swank clasp) (swank gray))
   #+mkcl '((swank mkcl) (swank gray))
+  #+mezzano '((swank mezzano) (swank gray))
   )
 
 (defparameter *implementation-features*
   '(:allegro :lispworks :sbcl :clozure :cmu :clisp :ccl :corman :cormanlisp
-    :armedbear :gcl :ecl :scl :mkcl :clasp))
+    :armedbear :gcl :ecl :scl :mkcl :clasp :mezzano))
 
 (defparameter *os-features*
   '(:macosx :linux :windows :mswindows :win32 :solaris :darwin :sunos :hpux
-    :unix))
+    :unix :mezzano))
 
 (defparameter *architecture-features*
-  '(:powerpc :ppc :x86 :x86-64 :x86_64 :amd64 :i686 :i586 :i486 :pc386 :iapx386
-    :sparc64 :sparc :hppa64 :hppa :arm :armv5l :armv6l :armv7l :arm64
+  '(:powerpc :ppc :ppc64 :x86 :x86-64 :x86_64 :amd64 :i686 :i586 :i486 :pc386 :iapx386
+    :sparc64 :sparc :hppa64 :hppa :arm :armv5l :armv6l :armv7l :arm64 :aarch64
     :pentium3 :pentium4
     :mips :mipsel
     :java-1.4 :java-1.5 :java-1.6 :java-1.7))
@@ -93,7 +98,7 @@
   #+lispworks (lisp-implementation-version)
   #+allegro   (format nil "~@{~a~}"
                       excl::*common-lisp-version-number*
-                      (if (eq 'h 'H) "A" "M")     ; ANSI vs MoDeRn
+                      (if (string= 'lisp "LISP") "A" "M")     ; ANSI vs MoDeRn
                       (if (member :smp *features*) "s" "")
                       (if (member :64bit *features*) "-64bit" "")
                       (excl:ics-target-case
@@ -103,7 +108,9 @@
                 (subseq s 0 (position #\space s)))
   #+armedbear (lisp-implementation-version)
   #+ecl (ecl-version-string)
-  #+clasp (clasp-version-string))
+  #+clasp (clasp-version-string)
+  #+mezzano (let ((s (lisp-implementation-version)))
+              (subseq s 0 (position #\space s))))
 
 (defun unique-dir-name ()
   "Return a name that can be used as a directory name that is
@@ -141,18 +148,20 @@ operating system, and hardware architecture."
 Return nil if nothing appropriate is available."
   (with-open-file (s (merge-pathnames "slime.el" *source-directory*)
                      :if-does-not-exist nil)
-    (loop with prefix = ";; Version: "
-          for line = (read-line s nil :eof)
-          until (eq line :eof)
-          when (string-starts-with line prefix)
-            return (subseq line (length prefix)))))
+    (when s
+      (loop with prefix = ";; Version: "
+            for line = (read-line s nil :eof)
+            until (eq line :eof)
+            when (string-starts-with line prefix)
+              return (subseq line (length prefix))))))
 
 (defun default-fasl-dir ()
   (merge-pathnames
    (make-pathname
     :directory `(:relative ".slime" "fasl"
                  ,@(if (slime-version-string) (list (slime-version-string)))
-                 ,(unique-dir-name)))
+                 ,(unique-dir-name)
+                 ,@(if *load-truename* (cdr (pathname-directory *load-truename*)))))
    (user-homedir-pathname)))
 
 (defvar *fasl-directory* (default-fasl-dir)
@@ -247,7 +256,7 @@ If LOAD is true, load the fasl file."
   '(swank-util swank-repl
     swank-c-p-c swank-arglists swank-fuzzy
     swank-fancy-inspector
-    swank-presentations swank-presentation-streams
+    swank-presentations
     #+(or asdf2 asdf3 sbcl ecl) swank-asdf
     swank-package-fu
     swank-hyperdoc
@@ -327,13 +336,16 @@ If LOAD is true, load the fasl file."
         (delete-package package)))))
 
 (defun init (&key delete reload load-contribs (setup t)
-                  (quiet (not *load-verbose*)))
+                  (quiet (not *load-verbose*))
+                  from-emacs)
   "Load SWANK and initialize some global variables.
 If DELETE is true, delete any existing SWANK packages.
 If RELOAD is true, reload SWANK, even if the SWANK package already exists.
 If LOAD-CONTRIBS is true, load all contribs
 If SETUP is true, load user init files and initialize some
 global variabes in SWANK."
+  (when from-emacs
+    (setf *started-from-emacs* t))
   (when (and delete (find-package :swank))
     (delete-packages (list-swank-packages)))
   (cond ((or (not (find-package :swank)) reload)
@@ -364,3 +376,20 @@ global variabes in SWANK."
               (collect-fasls (src-files *contribs*
                                         (contrib-dir *source-directory*))
                              (contrib-dir *fasl-directory*))))))
+
+(defmacro define-package (package &rest options)
+  "This is like CL:DEFPACKAGE but silences warnings and errors
+  signalled when the redefined package is at variance with the current
+  state of the package. Typically this situation occurs when symbols
+  are exported by calling EXPORT (as is the case with DEFSECTION) as
+  opposed to adding :EXPORT forms to the DEFPACKAGE form and the
+  package definition is subsequently reevaluated. See the section on
+  [package variance](http://www.sbcl.org/manual/#Package-Variance) in
+  the SBCL manual."
+  `(eval-when (:compile-toplevel :load-toplevel, :execute)
+     (locally
+         (declare #+sbcl
+                  (sb-ext:muffle-conditions sb-kernel::package-at-variance))
+       (handler-bind
+           (#+sbcl (sb-kernel::package-at-variance #'muffle-warning))
+         (cl:defpackage ,package ,@options)))))
